;*****************************************************************+*
;* LCD-Converter - Use a Standard Graphics LCD in an HP 4278A      *                                                  
;*-----------------------------------------------------------------*
;* The original LCD is a direct driven Sharp-LM2410J without RAM   *
;* and controller, using a 4-Bit Data Bus, and separate Pixel-,    *
;* Line-, Frame- and Toggle Clocks. The pixel data clock is about  *
;* 750 KHz. Currently the following code supports pixel clocks up  *
;* to 1 MHz, depending mainly on the speed of the Pixel Data Clock *
;* Interrupt Handler. The refresh rate of the Original Display is  *
;* about 75Hz, the following updates a standard T6963-driven LCD   *
;* with 240x128 Pixels with a rate of 25 Hz. Still much more, than *
;* the eyes can read... ;-)                                        * 
;*-----------------------------------------------------------------*
;* CPU: AVR 1284 at 20 MHz AVR Clock Frequency 	                   *
;*******************************************************************

.include "m1284Pdef.inc"

;***************************************
;* Input: LCD Signals from the HP4278A *
;*-------------------------------------*
;* Pixel Data Clock         = INT0     *
;* Frame (Vertical) Clock   = INT1     *
;* Line (Horizontal) Clock  = PCINT24  *
;* Frame Toggle Clock       = PCINT25  *
;*-------------------------------------*
;* 4 Bit Data Bus           = PC-3210  *
;***************************************

;*****************************
;* Output: 240x128 LCD Lines *
;*---------------------------*
;* Data Lines at PA-76543210 *
;*---------------------------*
;* Control Lines at PB-43210 *
;*---------------------------*
;*  B7 B6 B5  B4 B3 B2 B1 B0 *
;*  [--ISP--] RS CD CE RD WR *
;*****************************

;*******************************
;* Rotary Button PH12 at PC-76 *
;* Rotary Push Button at PC-5  *
;*******************************

;*****************
;* Debug Signals *
;*****************
.set INVERTCPA			= 0							; 0 = No, 1 = Yes
.set DEBUGMODE 			= 0  						; 0 = No, 1 = Yes
#define DEBS_SAMP 		4       					; Debug PD4 = Sample Cycle
#define DEBS_DISP		5 	     					; Debug PD5 = Display Cycle

;*************
;* Constants *
;*************
.equ GRAM1_BEG			= 0x1000					; Graphic RAM Page 1
.equ GRAM1_END			= 0x2000
.equ GRAM2_BEG			= 0x2000 					; Graphic RAM Page 2
.equ GRAM2_END			= 0x3000
.equ GRAM3_BEG			= 0x3000 					; Graphic RAM Page 3
.equ GRAM3_END			= 0x4000
.equ HPIX				= 240						; Horizontal Pixels
.equ VPIX				= 128						; Vertical Pixels
.equ TX_PIX_COL			= 6							; Text Mode: Pixel per Column
.equ GR_PIX_COL			= 8							; Graphic Mode: Pixel per Column
.equ PIX_ROW			= 8							; Pixel per Row
.equ TX_NUM_COL			= HPIX/TX_PIX_COL			; Text Mode: Number of Horizontal Chars (and Bytes)
.equ GR_NUM_COL			= HPIX/GR_PIX_COL			; Graphic Mode: Number of Horizontal Chars (and Bytes)
.equ NUM_ROW			= VPIX/PIX_ROW				; Number of Char Rows
.equ TXT_MEMBA			= 0							; Text Screen Base Address
.equ TXT_MEMSZ			= TX_NUM_COL*NUM_ROW		; Text Screen Size
.equ GRA_MEMBA			= TXT_MEMSZ					; Graphic Screen Base Address
.equ GRA_MEMSZ			= GR_NUM_COL*VPIX			; Graphic Screen Size
.equ BPL_OFFS			= 2							; Original LCD: 32 Bytes per Line?
.equ DEBOUNCE			= 8							; Button Debounce Ticks, about 104 ms at 13,3ms Frame Time
.equ PWM_MODE			= (1 << COM2B1) | \
						  (1 << WGM01) | \
						  (1 << WGM00)				; Fast PWM, Clear OC2B (PWM Output) on Compare
.equ PWM_PRSC			= (1 << CS21)				; Divide by 8, PWM at 9,765625 kHz (20 MHz Clock)

;******************
;* T6963 Hardware *
;******************
.equ CGRAM_BEG			= 0x1000					; Internal Char Generator RAM Address
.equ CGRAM_OFFS			= (CGRAM_BEG >> 11)			; Char Generator Offset
.equ T6963_STCX			= 0							; Command Status Bit
.equ T6963_STRW			= 1							; Read/Write Status Bit
.equ T6963_STAR			= 2							; Auto Read Status Bit
.equ T6963_STAW			= 3							; Auto Write Status Bit
.equ T6963_STA_RWC		= (1 << T6963_STCX) | \
						  (1 << T6963_STRW) 		; Combined RW and Command Status Bit Mask
.equ T6963_STA_AUW		= (1 << T6963_STAW) 		; Auto Write Status Bit Mask
.equ T6963_DatWR		= 0b00010010				; RS = 1, CD = 0, CE = 0, RD = 1, WR = 0
.equ T6963_CmdWR		= 0b00011010				; RS = 1, CD = 1, CE = 0, RD = 1, WR = 0
.equ T6963_CmdRD		= 0b00011001				; RS = 1, CD = 1, CE = 0, RD = 0, WR = 1
.equ T6963_Idle			= 0b00011111				; RS = 1, CD = 1, CE = 1, RD = 1, WR = 1

;***********************
;* T6963 Command Codes *
;***********************
#define SET_OFS_REG		0x22						; T6963 Offset Register
#define SET_ADR_PTR		0x24						; T6963 Address Pointer
#define SET_TXT_HOME	0x40						; T6963 Text Home Address
#define SET_TXT_AREA	0x41						; T6963 Text Area Size
#define SET_GR_HOME		0x42						; T6963 Graphics Home Address
#define SET_GRA_AREA	0x43						; T6963 Graphics Area Size
#define SET_MODE_OR		0x80						; T6963 Mode = OR
#define SET_MODE_TXAT	0x84						; T6963 Mode = Text Attribute
#define SET_TX1_GR0		0x94						; T6963 Text on / Graphics off
#define SET_TX0_GR1		0x98						; T6963 Text off / Graphics on
#define SET_TX1_GR1		0x9C						; T6963 Text on / Graphics on
#define SET_AU_WRITE	0xB0						; T6963 Start Auto Write Mode
#define AUTO_MO_END		0xB2						; T6963 End Auto Write Mode
#define DATA_WR_INC		0xC0						; T6963 Data Write and Increment
#define DATA_WR_NVA		0xC4						; T6963 Data Write

;*******************************
;* Global Register Definitions *
;*******************************
.def ProdLo				= r0						; MUL Instruction Result Low Byte
.def ProdHi				= r1						; MUL Instruction Result High Byte
.def SREGSave1			= r2						; SREG Save Register during Pixel Clock Interrupt
.def SREGSave2			= r3						; SREG Save Register during Frame or Timer Interrupt
.def SamNibb 			= r4						; The sampled Nibble from the original LCD-Bus
.def SamByte            = r5						; The sampled Byte = two sampled Nibbles
.def SamMask            = r6						; Sample Nibble Mask, toggled beetween 0x0F and 0xF0
.def RamPage			= r7						; Graphics Page (1/2) Flag
.def MenuMode			= r8						; Menu Mode Flag
.def Cycle      		= r9						; Sample Cycle Index
.def LastRot			= r10						; Last Rotary 
.def PushbCnt			= r11
.def RotDelta			= r12
.def Button				= r13

.def A  				= r16
.def B  				= r17
.def C  				= r18
.def D  				= r19
.def E	 				= r20
.def F  				= r21
.def G  				= r22
.def H  				= r23
.def I  				= r24
.def J 					= r25

;******************
;* IO Delay Macro *
;******************
.macro IODEL
			nop
			nop
			nop
			nop
			nop
.endmacro

;****************
;* SRAM Storage *
;****************
			.dseg
			.org	SRAM_START

RAMPWMVal:	.db		0								; 0..255: PWM Value to control LCD VEE Supply
RAMDifLim:	.db		0								; 0..255: Difference Limit for the Page Compare
RAMNoiLim:	.db		0								; 0: Noise limiter off, 1: Noise Limiter on
RAMInvLCD:	.db		0								; 0: Normal Display, 1: Inverted Display

 
;******************
;* EEProm Storage *
;******************
			.eseg
			.org	0x0000

EEPPWMVal:	.db		108								; 0..255: PWM Value to control LCD VEE Supply
EEPDifLim:	.db		32								; 0..255: Difference Limit for the Page Compare
EEPNoiLim:	.db		1								; 0: Noise limiter off, 1: Noise Limiter on
EEPInvLCD:	.db		0								; 0: Normal Display, 1: Inverted Display

;**************************
;* Code Entry Point Table *
;**************************
			.cseg
			.org	0x0000
			rjmp	Restart							; Main Program Entry Point
			.org	INT0addr
			rjmp	DataClkInt						; Original LCD Data Clock Handler
			.org	PCI3addr
			rjmp	FrameInt						; Original LCD Frame Toggle Interrupt Handler
			.org	OVF0addr
			rjmp	Timer0Int						; Timer 0 Overflow Interrupt Handler, Sample Encoder and Button


;******************************
;* Constants in Program Space *
;******************************
			.org	0x0040

EncSteps:	.db	    0,-1,1,0,0,0,0,0,0,0,0,0,0,1,-1,0
Int2ATab:	.db		100,10,1,0

MenuLine_0:	.db		 1,  0,  0, "            LCD Service Menu            ", 0
MenuLine_1:	.db		 4, 30, 34, "    Screen Contrast Adjust              ", 0
MenuLine_2:	.db		 6, 30, 34, "    Screen Noise Reduction              ", 0
MenuLine_3:	.db		 8, 30, 34, "    Screen Noise Limitation             ", 0
MenuLine_4:	.db		10, 30, 34, "    Inverted Display Mode               ", 0
MenuLine_5:	.db		13,  7, 29, "        Save Changes and exit           ", 0
MenuLine_6:	.db		14,  6, 30, "       Ignore Changes and exit          ", 0

MenTxtTab:	.dw		MenuLine_0, MenuLine_1, MenuLine_2, MenuLine_3, MenuLine_4, MenuLine_5, MenuLine_6, 0
MenVarTab:	.dw		0,  RAMPWMVal, RAMDifLim, RAMNoiLim, RAMInvLCD, 0, 0



;*****************
;* Wait Routine  *
;*---------------*
;* Input: <void> *
;*****************
Wait2ms:	push	A
			push	B
			ldi		A, 160							; About 2 ms at 20 MHz
Wait2ms_0:	clr		B
Wait2ms_1:	inc		B
			brne	Wait2ms_1						; 12,8 us at 20 MHz
			dec		A
			brne	Wait2ms_0
			pop		B
			pop		A
			ret

;***********************************
;* Initialize all Ports, Reset LCD *
;*---------------------------------*
;* Input: <void>                   *
;***********************************
InitPorts:	push	A
			ldi		A, 0b00000000
			out		DDRA, A							; Port A-76543210 = Input (LCD-Data Bus)
			ldi		A, 0b00111111
			out		DDRB, A							; Port B-54321 = LCD Control Signals
			out		PORTB, A	 					; All LCD Contol Signals = 1
			rcall	wait2ms
			ldi     A, 0b00101111					; RS = 0, Apply LCD Reset
			out		PORTB, A	
			rcall	wait2ms
			ldi     A, 0b00111111					; RS = 1, Remove LCD Reset
			out		PORTB, A	
			ldi		A, 0b00000000
			out		DDRC, A							; Port C-3210 = Sample Data Input
			ldi		A, 0b11110000					; Port C-7654 = Pull Up on
			out		PORTC, A	
			ldi		A, 0b11110000					; Port D7 = LCD FS, D6 = PWM D-54 = Debug Pins 
			out		DDRD, A	
			ldi		A, 0b00000000					; All Test Signals = 0, FS = Port D7 = 0
			out		PORTD, A	
			pop		A
			ret

;**************************************************************
;* Write Command or Data Byte/Word to the Graphics Controller *
;*------------------------------------------------------------*
;* Input: A = Command or Data Value                           *
;**************************************************************
WrComm:    	push	B
			push	A								; Command Value
			ldi		B, T6963_CmdWR					; Mode = Write Command
			rjmp	WrGraco
WrData:    	push	B
			push	A								; Data Value
			ldi		B, T6963_DatWR					; Mode = Write Data
WrGraco:	ldi		A, 0b00000000
			out     DDRA, A							; Port A = Input
			ldi		A, T6963_CmdRD
        	out     PORTB, A			       		; RS = 1, CD = 1, CE = 0, RD = 0, WR = 1
			IODEL
WrGraco_0:	in		A, PINA							; Get Status Byte
			andi	A, T6963_STA_RWC				; Check Command and Data R/W Flags
			cpi		A, T6963_STA_RWC
			brne	WrGraco_0						; Wait for RW and Execution Capability
			ldi		A, T6963_Idle
        	out     PORTB, A			       		; RS = 1, CD = 1, CE = 1, RD = 1, WER = 1
        	ldi		A, 0b11111111
			out     DDRA, A							; Port A = Output
			pop		A
			out		PORTA, A						; Write Command Byte
         	out     PORTB, B		       			; RS = 1, CD = 1, CE = 0, RD = 1, WR = 0
			IODEL
			ldi		B, T6963_Idle
        	out     PORTB, B			       		; RS = 1, CD = 1, CE = 1, RD = 1, WER = 1
			pop		B
			ret

;**************************************************************
;* Auto Write Command or Data Byte to the Graphics Controller *
;*------------------------------------------------------------*
;* Input: A = Command or Data Value                           *
;**************************************************************
AuWrComm:  	push	B
			push	A
			ldi		B, T6963_CmdWR					; Mode = Write Command
			rjmp	AuWrGra
AuWrData:  	push	B
			push	A
			ldi		B, T6963_DatWR					; Mode = Write Data
AuWrGra:	ldi		A, 0b00000000
			out     DDRA, A							; Port A = Input
 			ldi		A, T6963_CmdRD
        	out     PORTB, A			       		; RS = 1, CD = 1, CE = 0, RD = 0, WR = 1
			IODEL
AuWrGra_0:	in		A, PINA							; Get Status Byte
			sbrs    A, T6963_STAW					; Check Auto Data Write Flag
			rjmp	AuWrGra_0						; Wait for Auto Write Capability
			ldi		A, T6963_Idle
        	out     PORTB, A			       		; RS = 1, CD = 1, CE = 1, RD = 1, WER = 1
        	ldi		A, 0b11111111
        	out     DDRA, A							; Port A = Output
			pop		A
			out		PORTA, A						; Auto Write Data Byte
        	out     PORTB, B		       			; RS = 1, CD = 0, CE = 0, RD = 1, WR = 0
			IODEL
			ldi		B, T6963_Idle
        	out     PORTB, B			       		; RS = 1, CD = 1, CE = 1, RD = 1, WER = 1
			pop		B
			ret

;*****************************************************
;* Write 16 Bit Data Word to the Graphics Controller *
;*---------------------------------------------------*
;* Input: ZH:ZL = Data Word                          *
;*****************************************************
WrData16:	push	A
			mov		A, ZL							; Low Word
			rcall	WrData
			mov		A, ZH							; High Word
			rcall	WrData
			pop		A
			ret

;******************************************
;* Initialize the LCD Graphics Controller *
;*----------------------------------------*
;* Input: <void>                          *
;******************************************
InitGraCo:	push	A
			push	ZL
			push	ZH
			ldi		ZL, LOW(GRA_MEMBA)				; Set Graphics Home Addresss
			ldi		ZH, HIGH(GRA_MEMBA)
			rcall	WrData16
			ldi		A, SET_GR_HOME
			rcall	WrComm
			ldi		ZL, LOW(TXT_MEMBA)				; Set Text Home Address
			ldi		ZH, HIGH(TXT_MEMBA)
			rcall	WrData16
			ldi		A, SET_TXT_HOME
			rcall	WrComm
			ldi		ZL, LOW(CGRAM_OFFS)				; Set CharGen RAM Offset
			ldi		ZH, HIGH(CGRAM_OFFS)
			rcall	WrData16
			ldi		A, SET_OFS_REG
			rcall	WrComm
			pop		ZH
			pop		ZL
			pop		A
			ret

;*****************
;* Set Text Mode *
;*---------------*
;* Input: <void> *
;*****************
TextMode:	push	A
			push	ZL
			push	ZH
			rcall	InitGraCo
			sbi		PORTD, 7						; LCD FS = 1, Column Width = 6 Pixel
			ldi		ZL, LOW(TX_NUM_COL)				; Set Graphics (Attributes )Horzontal Width 
			ldi		ZH, HIGH(TX_NUM_COL)
			rcall	WrData16
			ldi		A, SET_GRA_AREA
			rcall	WrComm
			ldi		ZL, LOW(TX_NUM_COL)				; Set Text Area Horizontal Width
			ldi		ZH, HIGH(TX_NUM_COL)
			rcall	WrData16
			ldi		A, SET_TXT_AREA
			rcall	WrComm
			ldi		A, SET_TX1_GR1					; Text Display ON, Graphics Display (Attributes) ON
			rcall	WrComm
			ldi		A, SET_MODE_TXAT				; Set Text Attribute Mode
			rcall	WrComm
			pop		ZH
			pop		ZL
			pop		A
			ret

;********************
;* Set Graphic Mode *
;*------------------*
;* Input: <void>    *
;********************
GraphMode:	push	A
			push	ZL
			push	ZH
			rcall	InitGraCo
			cbi		PORTD, 7						; LCD FS = 0, Column Width = 8 Pixel
			ldi		ZL, LOW(GR_NUM_COL)				; Set Graphics Horzontal Width 
			ldi		ZH, HIGH(GR_NUM_COL)
			rcall	WrData16
			ldi		A, SET_GRA_AREA
			rcall	WrComm
			ldi		A, SET_TX0_GR1					; Text Display OFF, Graphics Display ON
			rcall	WrComm
			ldi		A, SET_MODE_OR					; Set OR Mode
			rcall	WrComm
			pop		ZH
			pop		ZL
			pop		A
			ret

;***************************
;* Text Goto Column / Line *
;*-------------------------*
;* Input: ZL = Column      *
;*		  ZH = Line        *
;***************************
TxtGotoXY:	push	A
			push	ZL
			push	ZH
			ldi		A, TX_NUM_COL
			mul		ZH, A							; r1:r0 = NUM_COL*Line
			ldi		ZH, 0x00
			add		ZL, ProdLo
			adc		ZH, ProdHi						; ZH:ZL = NUM_COL*Line+Column
			rcall	WrData16
			ldi		A, SET_ADR_PTR					; Set Address Pointer
			rcall	WrComm
			pop		ZH
			pop		ZL
			pop		A
			ret

;***************************************
;* Text Set Attribute at Column / Line *
;*-------------------------------------*
;* Input: A  = Attribute Byte          *
;*		  ZL = Column                  *
;*		  ZH = Line                    *
;***************************************
TxtAttrXY:	push	A
			push	ZL
			push	ZH
			push	A
			ldi		A, TX_NUM_COL
			mul		ZH, A							; ProdLo:ProdHi = (NUM_COL*Line)
			ldi		ZH, 0x00
			add		ZL, ProdLo
			adc		ZH, ProdHi						; ZH:ZL = ((NUM_COL*Line)+Column)
			ldi		A, LOW(TXT_MEMSZ)
			add		ZL, A	
			ldi		A, HIGH(TXT_MEMSZ)
			adc		ZH, A	
			rcall	WrData16
			ldi		A, SET_ADR_PTR					; Set Address Pointer
			rcall	WrComm
			pop		A								; A = Attribute Byte
			rcall	WrData							; Write Attribute Byte
			ldi		A, DATA_WR_NVA						
			rcall	WrComm							; Write without changing Adress
			pop		ZH
			pop		ZL
			pop		A
			ret

;*******************************
;* Write Single Text Character *
;*-----------------------------*
;* Input: A = Char Code        *
;*******************************
WriteChar:	push	A
			subi	A, 0x20
			rcall	WrData							; Write Character
			ldi		A, DATA_WR_INC
			rcall	WrComm							; Write and Increment Address
			pop		A
			ret

;***************************************************
;* Write Zero terminated String from Program Space *
;*-------------------------------------------------*
;* Input: ZH:ZL = String Address                   *
;***************************************************
WrPMString:	push	A
			push	ZL
			push	ZH
			ldi		A, SET_AU_WRITE					; Data Write and Increment
			rcall	WrComm
WrPMStr_0:	lpm		A, Z+							; A = Next Character
			tst		A							
			breq	WrPMStr_1						; Skip if at String End
			subi	A, 0x20
			rcall	AuWrData
			rjmp	WrPMStr_0
WrPMStr_1:	ldi		A, AUTO_MO_END
			rcall	WrComm							; Exit from Auto Write Mode
			pop		ZH
			pop		ZL
			pop		A
			ret

;*********************************************
;* Write a unsigned Byte wide Integer Number *
;*-------------------------------------------*
;* Input: A = Unsigned Byte Value            *
;*********************************************
WriteUInt8:	push	A
			push	B
			push	C
			push	ZL
			push	ZH
			mov		B, A							; B = unsigned Byte Value
			ldi		ZL, LOW(Int2ATab << 1)
			ldi		ZH, HIGH(Int2ATab << 1)			; Z = Decade Table
WrUint8_0:	lpm 	C, Z+							; Load Next Decade Subtrahend 
			tst		C							
			breq	WrUInt8_2						; Skip if all Done
			clr		A								; Digit = 0
WrUInt8_1:	inc		A								; Increment Digit
			sub		B, C							; Subtract Decade Value
			brcc	WrUInt8_1						; Skip if Value >= 0
			add		B, C							; Restore Data Value
			subi	A, -('0'-1)						; Convert the Digit to an ASCII Character
			rcall	WriteChar						; Write Digit Character
			rjmp	WrUINt8_0						; Loop until Done
WrUInt8_2:	pop		ZH
			pop		ZL
			pop		C
			pop		B
			pop		A
			ret

;********************************
;* Clear Text or Graphic Screen *
;*------------------------------*
;* Input: <void>                *
;********************************
ClrTxtScr:  push	A
			push	XL
			push	XH
			push	ZL
			push	ZH
			ldi		ZL, LOW(TXT_MEMBA)				; Address Pointer = Graphics Home Address
			ldi		ZH, HIGH(TXT_MEMBA)
			ldi		XL, LOW(TXT_MEMSZ << 1)			; Text Screen Size in Byte multipled by 2
			ldi		XH, HIGH(TXT_MEMSZ << 1)		; Second Text Area = Attribute Area
			rjmp	ClrScr_0
ClrGraScr:	push	A
			push	XL
			push	XH
			push	ZL
			push	ZH
			ldi		ZL, LOW(GRA_MEMBA)				; Address Pointer = Graphics Home Address
			ldi		ZH, HIGH(GRA_MEMBA)
			ldi		XL, LOW(GRA_MEMSZ)				; Graphic Screen Size in Byte
			ldi		XH, HIGH(GRA_MEMSZ)
ClrScr_0:	rcall	WrData16
			ldi		A, SET_ADR_PTR					; Set Address Pointer
			rcall	WrComm
			ldi		A, SET_AU_WRITE					; Data Write and Increment
			rcall	AuWrComm
ClrScr_1:	ldi		A, 0x00
			rcall	AuWrData						; Clear all Screen Bytes
			sbiw	XH:XL, 1
			brne	ClrScr_1
			ldi		A, AUTO_MO_END
			rcall	AuWrComm						; Exit from Auto Write Mode
			pop		ZH
			pop		ZL
			pop		XH
			pop		XL
			pop		A
			ret

;*****************************************************************
;* Update the LCD Graphics Memory from a Controller Memory Block *
;*---------------------------------------------------------------*
;* Input: XH:XL = Controller Memory Start Address                *
;*****************************************************************
UpdateLCD:	push	A
			push	B
			push	C
			push	D
			push	ZL
			push	ZH
			clr		D
			ldi		ZL, LOW(RAMInvLCD)
			ldi		ZH, HIGH(RAMInvLCD)
			ld		A, Z
			tst		A
			breq	UpdLCD_0						; Skip if not inverted Mode
			com		D								; D = 0x00 or 0xFF
UpdLCD_0:	ldi		ZL, LOW(GRA_MEMBA)				; Address Pointer = Graphics Home Address
			ldi		ZH, HIGH(GRA_MEMBA)
			rcall	WrData16
			ldi		A, SET_ADR_PTR					; Set Address Pointer
			rcall	WrComm
			ldi		A, SET_AU_WRITE					; Data Write and Increment
			rcall	AuWrComm
			ldi		B, VPIX
UpdLCD_1:	ldi		C, GR_NUM_COL
UpdLCD_2:	ld		A, X+							; Next 8 Pixel
			eor 	A, D
			rcall	AuWrData						; Write to LCD
			dec		C								; Horizontal Pixel Loop
			brne	UpdLCD_2
			adiw	XH:XL, BPL_OFFS
			dec		B								; Vertical Lines Loop
			brne	UpdLCD_1
			ldi		A, AUTO_MO_END
			rcall	AuWrComm						; Exit from Auto Write Mode
UpdLCD_3:	pop		ZH
			pop		ZL
			pop		D
			pop		C
			pop		B
			pop		A
			ret

;*******************************************************************
;* Compare Graphics Page 1 and Graphic Page 2  (without Mask Area) *
;*-----------------------------------------------------------------*
;* Input:  A = Maximum Error Count                                 *
;* Output: CY = 1 if  Difference Limit reached, else CY = 0        *
;*******************************************************************
MemComp:	push	B
			push	C
			push	D
			push	E
			push	F
			push	G
			push	XL
			push	XH
			push	YL
			push	YH
			push	ZL
			push	ZH
			ldi		XL, LOW(GRAM1_BEG)				; Graphic Page 1
			ldi		XH, HIGH(GRAM1_BEG)
			ldi		YL, LOW(GRAM2_BEG)				; Graphic Page 2
			ldi		YH, HIGH(GRAM2_BEG)
			ldi		ZL, LOW(GRAM3_BEG)				; Compare Area Mask
			ldi		ZH, HIGH(GRAM3_BEG)
			ldi		G, 0x00							; Difference Count = 0
			ldi		B, VPIX
MemComp_0:	ldi		C, GR_NUM_COL
MemComp_1:	ld		D, X+							; Byte from Graphic Page 1
			ld		E, Y+							; Byte from Graphic Page 2
			ld		F, Z+							; Mask Value
			cpi		F, 0xFF
			breq	MemComp_2						; Skip if in non-Compare Area
			cp		E, D							; Else Compare both Graphic Pages
			breq	MemComp_2						; Skip if the Pages are identical
			inc		G
			cp		G, A							; Difference Limit reached?
			brsh	MemComp_3
MemComp_2:	dec		C								; Horizontal Pixel Loop
			brne	MemComp_1
			adiw	XH:XL, BPL_OFFS
			adiw	YH:YL, BPL_OFFS
			dec		B								; Vertical Lines Loop
			brne	MemComp_0
			clc										; Difference Limit not reached, CY = 0
			rjmp	MemComp_4
MemComp_3:	sec										; Difference Limit reached, CY = 1
MemComp_4:	pop		ZH
			pop		ZL
			pop		YH
			pop		YL
			pop		XH
			pop		XL
			pop		G
			pop		F
			pop		E
			pop		D
			pop		C
			pop		B
			ret

;**********************************
;* Generate the Compare Area Mask *
;*--------------------------------*
;* Input: <void>                  *
;**********************************
GenCmpMsk:	push	A
			push	B
			push	C
			push	XL
			push	XH
			ldi		XL, LOW(GRAM3_BEG)				; Mask in Graphic Page 3
			ldi		XH, HIGH(GRAM3_BEG)
			ldi		B, 0
GCMask_0:	ldi		C, 0
GCMask_1:	clr		A
			cpi		B, 47							; First Mask Area Top Y Position
			brlo    GCMask_3
			cpi		B, 85							; Second Mask Area Bottom Y Position
			brsh    GCMask_3
			cpi		B, 61							; First Mask Area Bottom Y Position
			brlo	GCMask_2
			cpi		B, 71							; Second Mask Area Top Y Position
			brlo	GCMask_3
GCMask_2:	cpi		C, 2							; Mask Area Left X Position
			brlo	GCMask_3
			cpi		C, 23							; Mask Area Right X Position
			brsh	GCMask_3
			com		A								; Store 0x00 (unmasked) or 0xFF (masked)
GCMask_3:	st		X+, A	
			inc		C								; Horizontal Pixel Loop
			cpi		C, GR_NUM_COL
			brlo	GCMask_1
			inc		B								; Vertical Lines Loop
			cpi		B, VPIX
			brlo	GCMask_0
			pop		XH
			pop		XL
			pop		C
			pop		B
			pop		A
			ret

;*******************************
;* Display Sampled Screen Data *
;*-----------------------------*
;* Input: <void>               *
;*******************************
DispSamp:	push	A
			push	XL
			push	XH
			push	ZL
			push	ZH
			ldi		ZL, LOW(RAMNoiLim)
			ldi		ZH, HIGH(RAMNoiLim)				
			ld		A, Z
			tst		A
			breq	DispSamp_0						; Skip if Noise Limiter deaktivated
			ldi		ZL, LOW(RAMDifLim)
			ldi		ZH, HIGH(RAMDifLim)
			ld		A, Z							; A = Noise Limit Value
			rcall 	MemComp							; Compare sampled Graphic Pages
			brcs	DispSamp_3						; Skip if too many Differences
DispSamp_0:	or		RamPage, RamPage
			brmi	DispSamp_1
			ldi		XL, LOW(GRAM1_BEG)				; Display Graphics Page 1
			ldi		XH, HIGH(GRAM1_BEG)
			rjmp	DispSamp_2
DispSamp_1:	ldi		XL, LOW(GRAM2_BEG)				; Display Graphics Page 2
			ldi		XH, HIGH(GRAM2_BEG)
DispSamp_2:	sbi		PORTD, DEBS_DISP				; Debug Signal: Display Cycle = 1
			rcall	UpdateLCD						; Display sampled Graphics Page
			cbi		PORTD, DEBS_DISP				; Debug Signal: Display Cycle = 0
DispSamp_3:	pop		ZH
			pop		ZL
			pop		XH
			pop		XL
			pop		A
			ret

;*****************************
;* Sample the Rotary Encoder *
;*****************************
SampRotary:	push	A
			push	ZL
			push	ZH
			ldi		A, 0b00110000
			and		LastRot, A						; Mask the Last Rotation Phase Bits
			in		A, PINC						 
			andi	A, 0b11000000					; Bit 7+6 = Rotary Encoder Phases
			or		A, LastRot						; Bit 7+6 = Current, Bit 5+4 = Last Phase
			swap	A								; Bit 3+2 = Current, Bit 1+0 = Last Phase
			mov		LastRot, A	
			rol		LastRot
			rol		LastRot							; LastRot Bit 5+4 = Current Phase
			ldi		ZL, LOW(EncSteps << 1)			; Use a Look up Table to get +/-1
			ldi		ZH, HIGH(EncSteps << 1)			; Values for valid Encoder steps
			add		ZL, A	
			ldi		A, 0x00
			adc		ZH, A							; Z = Address of Lookup Table Entry
			lpm		A, Z						
			add		RotDelta, A						; Accumulate +/-1 Steps
			pop		ZH
			pop		ZL
			pop		A
			ret

;**************************
;* Sample the Push Button *
;**************************
SampButt:	push	A
			sbic	PINC, 5							; Skip if Button pressed
			rjmp	SampButt_0						; Skip if Button not pressed
			inc		PushBCnt						; Accumulate Button Debounce Count
			ldi		A, DEBOUNCE
			cp		PushBCnt, A						; Compare Debounce Count to Limit
			brlo	SampButt_2						; Skip if Count < Debounce Limit
			brne	SampButt_1						; Skip if Count > Debounce Limit
			ldi		A, 0b00000001
			mov		Button, A						; Mark Button pressed if Count == Limit
			pop		A
			ret
SampButt_0: tst		PushBCnt						
			breq	SampButt_2						; Skip if Debound Count is already 0
SampButt_1:	dec		PushBCnt						; Stop Debounce Count at Debounce Limit
SampButt_2:	pop		A
			ret

;**********************************
;* Wait until Pushbutton released *
;**********************************
WaitPBRel:	clr		Button							; Clear Button Flag
			tst		PushBCnt
			brne	WaitPBRel						; Wait Until Debounce Count = 0
			clr		RotDelta						; Ignore Rotations while Button pressed
			ret

;********************************					; Cycles	Sum-Cycles
;* Data Clock Interrupt Handler *					;---------------------------------------------
;********************************           		; <5>     	5		IRQ-Overhead-Time
DataClkInt:	in		SamNibb, PINC					; <1>     	7		Sample as early as possible
			in		SREGSave1, SREG					; <1>		6		Save Flags before the following "com"
			com		SamMask							; <1>    	8
			brmi	DCInt_0							; <1 2>   	9/10
			and		SamNibb, SamMask				; <1>     	10
			or		SamByte, SamNibb				; <1>    	11
			st		Y+, SamByte						; <2>    	13
			out		SREG, SREGSave1					; <1>		14
			reti									; <5>    	19		Total: 950ns at 20 MHz Clock Speed
DCInt_0:	mov		SamByte, SamNibb				; <1>    	11
			swap	SamByte							; <1>    	12
			and		SamByte, SamMask				; <1>	 	13
			out		SREG, SREGSave1					; <1>		14
			reti									; <5>    	19		Total: 950ns at 20 MHz Clock Speed

;**********************************
;* Frame Toggle Interrupt Handler *
;**********************************
FrameInt:	in		SREGSave2, SREG					; Save the Status Flags
			rcall	SampButt						; Sample the Push Button
			tst		MenuMode
			brne	FraInt_4						; Exit from Interrupt if in Menu Mode
			inc		Cycle							; Else Advance Cycle Index
			ldi		A, 0x01
			cp		Cycle, A						; Cycle = 1?
			breq	FraInt_1						; Skip if in Sample Cycle
			inc		A
			cp		Cycle, A						; Cycle = 2?
			brne	FraInt_4						; Skip if not in Display Cycle
			ldi		A, 0b00000000
			out		EIMSK, A						; Disable Int0 (Data Clock)
			sts		PCMSK3, A						; Disable PCINT25 (Frame Toggle Clock)
			cbi		PORTD, DEBS_SAMP				; Debug Signal: Sample Cycle = 0
			rcall	DispSamp
			tst		Button							; Check Button
			breq	FraInt_0						; Skip if Button not pressed
			mov		MenuMode, Button				; Else Set menu Mode to 0x01
			clr		Button							; Clear the Button pressed Flag
FraInt_0:	clr		Cycle							; Restart with Sampling Cycle
			ldi		A, 0b00000010
			sts		PCMSK3, A						; Enable PCINT25 (Frame Toggle Clock)
			rjmp	FraInt_4						; Exit from Interrupt
FraInt_1:	com		RamPage							; Toggle the Graphics Page Flag
			brmi	FraInt_2
			ldi		YL, LOW(GRAM1_BEG-1)			; Sample to Graphics Page 1
			ldi		YH, HIGH(GRAM1_BEG-1)
			rjmp	FraInt_3
FraInt_2:	ldi		YL, LOW(GRAM2_BEG-1)			; Sample to Graphics Page 2
			ldi		YH, HIGH(GRAM2_BEG-1)
FraInt_3:	ldi		A, 0xF0							; Preset the Nibble Mask
			mov		SamMask, A	
			sbi		PORTD, DEBS_SAMP				; Debug Signal: Sample Cycle = 1
			ldi		A, 0b00000001
			out		EIMSK, A						; Enable Int0 (Data Clock)
FraInt_4:	out		SREG, SREGSave2					; Restore the Status Flags
			reti

;*****************************
;* Timer 0 Interrupt Handler *
;*****************************
Timer0Int:	in		SREGSave2, SREG					; Save the Status Flags	
			sbi		PORTD, DEBS_SAMP				; Debug Signal: Sample Cycle = 1
			rcall	SampButt						; Sample the Menu Button
			rcall	SampRotary						; Sample the Rotary Encoder
			cbi		PORTD, DEBS_SAMP				; Debug Signal: Sample Cycle = 0
			out		SREG, SREGSave2					; Restore the Status Flags
			reti

;****************************************
;* Read a Byte from the internal EEProm *
;---------------------------------------*
;* Input:  ZH:ZL = EEProm Address       *
;* Output: A     = Byte from EEprom     * 
;****************************************
ReadEEP:	sbic	EECR, EEPE						; Skip if prevous Write completed
			rjmp	ReadEEP							; Else continue waiting
			out		EEARH, ZH						;
			out		EEARL, ZL						; EEARH:EEARL = EEprom Address
			sbi		EECR, EERE						; Init EEProm Read Cycle
			in		A, EEDR							; Read EEProm Data
			ret

;****************************************
;* Write a Byte to the internal EEProm  *
;---------------------------------------*
;* Input:  ZH:ZL = EEProm Address       *
;*         A     = Byte to Write        * 
;****************************************
WriteEEP:	sbic	EECR, EEPE						; Skip if prevous Write completed
			rjmp	WriteEEP						; Else continue waiting
			out		EEARH, ZH						;
			out		EEARL, ZL						; EEARH:EEARL = EEprom Address
			out		EEDR, A							; Write Data
			sbi		EECR, EEMPE						; Init EEProm Read Cycle
			sbi		EECR, EEPE						; Init Write EEProm Cycle
			ret

;***********************
;* Read all EEP Values *
;----------------------*
;* Input:  <viod>      *
;***********************
ReadAllEEP:	push	A
			push	XL
			push	XH
			push	ZL
			push	ZH
			ldi		ZL, LOW(EEPPWMVal)
			ldi		ZH, HIGH(EEPPWMVal)
			rcall	ReadEEP
			ldi		XL, LOW(RAMPWMVal)
			ldi		XH, HIGH(RAMPWMVal)
			st		X, A
			ldi		ZL, LOW(EEPDifLim)
			ldi		ZH, HIGH(EEPDifLim)
			rcall	ReadEEP
			ldi		XL, LOW(RAMDifLim)
			ldi		XH, HIGH(RAMDifLim)
			st		X, A
			ldi		ZL, LOW(EEPNoiLim)
			ldi		ZH, HIGH(EEPNoiLim)
			rcall	ReadEEP
			ldi		XL, LOW(RAMNoiLim)
			ldi		XH, HIGH(RAMNoiLim)
			st		X, A
			ldi		ZL, LOW(EEPInvLCD)
			ldi		ZH, HIGH(EEPInvLCD)
			rcall	ReadEEP
			ldi		XL, LOW(RAMInvLCD)
			ldi		XH, HIGH(RAMInvLCD)
			st		X, A
			pop		ZH
			pop		ZL
			pop		XH
			pop		XL
			pop		A
			ret

;***********************
;* Save all EEP Values *
;----------------------*
;* Input:  <viod>      *
;***********************
SaveAllEEP:	push	A
			push	XL
			push	XH
			push	ZL
			push	ZH
			ldi		XL, LOW(RAMPWMVal)
			ldi		XH, HIGH(RAMPWMVal)
			ld		A, X
			ldi		ZL, LOW(EEPPWMVal)
			ldi		ZH, HIGH(EEPPWMVal)
			rcall	WriteEEP
			ldi		XL, LOW(RAMDifLim)
			ldi		XH, HIGH(RAMDifLim)
			ld		A, X
			ldi		ZL, LOW(EEPDifLim)
			ldi		ZH, HIGH(EEPDifLim)
			rcall	WriteEEP
			ldi		XL, LOW(RAMNoiLim)
			ldi		XH, HIGH(RAMNoiLim)
			ld		A, X
			ldi		ZL, LOW(EEPNoiLim)
			ldi		ZH, HIGH(EEPNoiLim)
			rcall	WriteEEP
			ldi		XL, LOW(RAMInvLCD)
			ldi		XH, HIGH(RAMInvLCD)
			ld		A, X
			ldi		ZL, LOW(EEPInvLCD)
			ldi		ZH, HIGH(EEPInvLCD)
			rcall	WriteEEP
			pop		ZH
			pop		ZL
			pop		XH
			pop		XL
			pop		A
			ret

;*************************************************
;* Return Service Menu Data for a specifed Index *
;------------------------------------------------*
;* Input:  A     = Menu Index                    *
;* Output: XL,XH = String X,Y Position           *
;*         YL,YH = Left/Right Marker X Position  *       
;*         ZH:ZL = String Address                * 
;*         CY = 1 at Table End, else CY = 0      * 
;*************************************************
GetMenData:	ldi		ZL, LOW(MenTxtTab << 1)
			ldi		ZH, HIGH(MenTxtTab << 1)		; ZH:ZL = Menu Table
			lsl 	A								; Adjust for Word Size Entries
			add		ZL, A							
			clr		A
			adc		ZH, A							; ZH:ZL = Indexed Menu Table Entry
			lpm		YL, Z+
			lpm		YH, Z							; YH:YL = Menu Data Address
			mov		A, YL
			or		A, YH							; Check for Zero
			sec										; CY = 1
			breq	GetMenDa_0						; Skip if at End of Table
			mov		ZL, YL	
			mov		ZH, YH							; ZH:ZL = Menu Data Address
			lsl		ZL
			rol		ZH								; Adjust for Word Size Entries
			ldi		XL, 0x00						; XL = String XPos = 0
			lpm		XH, Z+							; XH = String YPos
			lpm		YL, Z+							; YL = Left Marker XPos
			lpm		YH, Z+							; YH = Right Marker XPos
			clc										; CY = 0
GetMenDa_0:	ret

;******************************************
;* Draw the Srvice Menu Marker Characters *
;-----------------------------------------*
;* Input: A  = Menu Index                 *
;*        B  = Mark On (1) / Off (0) Flag *
;******************************************
DrawMark:	push	A
			push	ZL
			push	ZH
			rcall	GetMenData
			mov		A, XH							; A = Y Position
			push	A								; Save Y Position
			mov		ZL, YL							; ZL = Left Marker X Position
			mov		ZH, A							; ZH = Left Marker Y Position
			rcall	TxtGotoXY						
			ldi		A, '['
			tst		B								; B == 0: Draw Space, else Draw marker
			brne	DrawMark_0
			ldi		A, ' '							
DrawMark_0:	rcall	WriteChar						; Draw Left Marker
			mov		ZL, YH							; ZL = Right Marker X Position
			pop		ZH								; ZH = Right Marker Y Position
			rcall	TxtGotoXY
			ldi		A, ']'
			tst		B								; B == 0: Draw Space, else Draw marker
			brne	DrawMark_1
			ldi		A, ' '							
DrawMark_1:	rcall	WriteChar						; Draw Left Marker
			pop		ZH
			pop		ZL
			pop		A
			ret

;******************************************
;* Set the Srvice Menu Marker Attributes  *
;-----------------------------------------*
;* Input: A  = Menu Index                 *
;*        B  = Marker Char Attribute      *
;******************************************
MarkAttr: 	push	A
			rcall	GetMenData
			mov		ZL, YL							; ZL = Left Mark X-Position
			mov		ZH, XH							; ZH = Y-Position
			mov		A, B							; A = Attribute
			rcall	TxtAttrXY
			mov		ZL, YH							; ZL = Right Mark X-Position
			mov		ZH, XH							; ZH = Y-Position
			mov		A, B							; A = Attribute
			rcall	TxtAttrXY
			pop		A
			ret

;****************************************
;* Change and Display the Menu Variable *
;---------------------------------------*
;* Input: A  = Menu Index               *
;*        B  = Adjust Value             *
;****************************************
CDMenVar:	rcall	GetMenData
			mov		ZH, XH							; Line
			mov		ZL, YL							; Column
			inc		ZL
			rcall	TxtGotoXY
			mov		A, C
			ldi		ZL, LOW(MenVarTab << 1)
			ldi		ZH, HIGH(MenVarTab << 1)
			lsl 	A								; Adjust for Word Size Entries
			add		ZL, A							
			clr		A
			adc		ZH, A							; ZH:ZL = Indexed Menu Table Entry
			lpm		YL, Z+
			lpm		YH, Z							; YH:YL = Menu Variable Address
			ld		A, Y
			subi	A, 0x80
			add		A, B
			brvc	CDMenVar_0
			ldi		A, 0x7F
			sbrc	B, 7
			ldi		A, 0x80
CDMenVar_0:	subi	A, 0x80
			st		Y, A
			push	A	
			rcall	WriteUInt8
			pop		A
			ret

;***************************
;* Handle the Service Menu *
;***************************
SrvMenu:	ldi		A, 0b00000000
			out		EIMSK, A						; Disable Int0 (Data Clock)
			sts		PCMSK3, A						; Disable PCINT25 (Frame Toggle Clock)
			ldi		A, (1 << WGM01) 				; Mode = CTC
			out		TCCR0A, A	
			ldi		A, (1 << CS02)					; Prescaler = 256, Timer Clock = 78,125 kHz
			out		TCCR0B, A	
			ldi		A, 78
			out		OCR0A, A						; Timer Interrupt Rate = 1 kHz
			ldi		A, (1 << OCIE0A)
			sts		TIMSK0, A						; Enable Timer 0 Output Compare A Match Interrupt
			sei										; Enable Interrupts

			rcall	ReadAllEEP						; Read All EEP Values to the SRAM

			clr		C								; C = Menu Index
DispMenu:	mov		A, C
			rcall	GetMenData
			brcs	WalkMenu						; Skip if Bottom of Menu reached
			push	ZL
			push	ZH
			mov		ZH, XH
			mov		ZL, XL
			rcall	TxtGotoXY						; Set Text X,Y Position
			pop		ZH
			pop		ZL
			rcall	WrPMString						; Write Menu String

			cpi		C, 0x01
			brlo	DispMenu_0
			cpi		C, 0x05
			brsh	DispMenu_0

			mov		A, C
			clr		B
			rcall	CDMenVar



DispMenu_0:	inc		C								; Next Menu Index
			rjmp	DispMenu						; Loop Until Done

WalkMenu:	ldi		C, 0x01
			rjmp	WalkMen_3

WalkMen_0:	tst		Button
			breq	WalkMen_1						; Skip if Button not pressed
			rcall	WaitPBRel						; Wait until Push Button released
			

			cpi		C, 0x05
			breq	SaveExit
			cpi		C, 0x06
			breq	Terminate


			mov		A, C
			ldi		B, 0x08
			rcall	MarkAttr

			clr		RotDelta
xxx:		tst		RotDelta
			breq	xxx_1
			mov		A, C
			mov		B, RotDelta
			clr		RotDelta
			rcall	CDMenVar

			cpi		C, 0x01
			brne	xxx_1
			sts		OCR2B, A
xxx_1:

			sbrs	Button, 0						; Skip if Button is pressed
			rjmp	xxx								; Else skip to check Rotary Count
			rcall	WaitPBRel						; Wait until Push Button released

			mov		A, C
			ldi		B, 0x00
			rcall	MarkAttr


WalkMen_1:	tst		Rotdelta
			breq	WalkMen_0
			
			mov		A, C
			clr		B
			rcall	DrawMark						; Remove Markers

			add		C, RotDelta
			clr		RotDelta
			ldi		A, 0x01
			cp		C, A	
			brlt	WalkMen_2
			ldi		A, 0x06
			cp		C, A	
			brlt	WalkMen_3
WalkMen_2:	mov		C, A		

WalkMen_3:	mov		A, C
			ldi		B, 0x01
			rcall	DrawMark						; Place Markers

			rjmp	WalkMen_0

SaveExit:	rcall	SaveAllEEP

Terminate:	ret

;***********************
;* Restart Entry Point *
;***********************
Restart:	cli										; Disable Interrupts
			wdr										; Watchdog Reset
			in		A, MCUSR
			andi	A, ~(1 << WDRF)					; Clear the Watchdog Reset Flag
			out		MCUSR, A	
			lds		A, WDTCSR
			ori 	A, (1 << WDCE) | (1 << WDE)		; Set Simultanous the Watch Dog Change enable
			sts		WDTCSR,	A						; Bit AND the Watch Dog Enable Bit
			ldi		A, (0 << WDE)					
			sts		WDTCSR, A						; Stop the Watch dog


			ldi		XL, LOW(SRAM_START)				; Start Adress of the Contoller Memory
			ldi		XH, HIGH(SRAM_START)
			ldi		YL, LOW(SRAM_SIZE)				; Size of the Controller Memory
			ldi		XH, HIGH(SRAM_SIZE)
			ldi     A, 0x00
ClearRam:	st		X+, A							; Clear all RAM Bytes
			sbiw	YH:YL, 1						; Decrement Size Count
			brne	ClearRam
			ldi     A, LOW(RAMEND) 					; Init the Stack Pointer
			out     SPL, A	
        	ldi     A, HIGH(RAMEND)
        	out     SPH, A	
			ldi		A, 0x00
			mov		Cycle, A						; Init Cycle Index
			mov		RamPage, A						; Init Ram Page
			mov		MenuMode, A						; Start in Normal Display Mode
			mov		PushBCnt, A						; Push Button Debounce Count = 0
			mov		Button, A						; Button not pressed
			rcall	InitPorts						; Initialize Ports

			rcall	ReadAllEEP

			ldi		A, PWM_MODE						; PWM Mode for Contrast Voltage Adjust
			sts		TCCR2A, A	
			ldi		A, PWM_PRSC						; Set PWM Frequency
			sts		TCCR2B, A	
			ldi		ZL, LOW(RAMPWMVal)
			ldi		ZH, HIGH(RAMPWMVal)
			ld		A, Z
			sts		OCR2B, A						; Generate PWM at PD-6 / OCR2B

			rcall	GraphMode						; Init Graphic Mode
			rcall	ClrGraScr						; Clear Graphic Screen
			rcall	GenCmpMsk						; Generate the Compare Area Mask
			ldi		A, 0b00001000
			sts		PCICR, A						; Enable Pin Change Interrupt 3
			ldi		A, 0b00000010
			sts		PCMSK3, A						; Enable PCINT24
			sei										; Enable Interrupts
			ldi		A, 0b00000010
			sts		EICRA, A						; INT0 = Data Clock Int, falling Edge
RunNormal:	sbrs	MenuMode, 0						; Skip if in Menu Mode
			rjmp	RunNormal						; Else stay in normal Display Mode
			rcall	WaitPBRel						; Wait until Push Button released
			cli										; Disable Interrupts
			rcall	TextMode						; Switch to Text Mode
			rcall	ClrTxtScr						; Clear Text Screen
			rcall	SrvMenu							; Handle the Service Menu
  			cli										; Disable Interrupts
			lds		A, WDTCSR      				
  			ori		A, (1 << WDE)				
 			sts		WDTCSR, A						; Set the Watch Dog Enable Bit
Stop:		rjmp	Stop							; Wait for the Watch Dog Reset





